@@ -44,8 +44,10 @@
 #' @md
 processDocument <- function(path) {
   doc <- list()
-  doc$image <- readPNGBinary(path)
-  doc$thin <- thinImage(doc$image)
+  # load image as matrix
+  doc$image <- readPNGBinary(path) 
+  # load writing as 1-column matrix of index locations of black pixels
+  doc$thin <- thinImage(doc$image)  
   doc$process <- processHandwriting(doc$thin, dim(doc$image))
   doc$docname <- basename(path)
   doc$docname <- stringr::str_replace(doc$docname, ".png", "")
@@ -83,16 +85,18 @@ processDocument <- function(path) {
 #' @md
 processHandwriting <- function(img, dims) {
   value <- from <- to <- nodeOnlyDist <- man_dist <- euc_dist <- pen_dist <- NULL
-
+  
   # Next, we have to follow certain rules to find non intersection breakpoints.
   
+  # Starting Processing ----
   message("Starting Processing...")
-  # convert thinned image from list of pixel indices to binary matrix: 0 for
+  # convert thinned handwriting from list of pixel indices to binary matrix: 0 for
   # handwriting and 1 elsewhere
   indices <- img  # thinned image as list of pixel indices
   img <- matrix(1, nrow = dims[1], ncol = dims[2])
   img[indices] <- 0
-
+  
+  # Getting Nodes ----
   message("Getting Nodes...", appendLF = FALSE)
   # output: nodeList[[1]] = all nodes
   # output: nodeList[[2]] = indices of nodes with 3 or more connected edges and indices of 2x2 nodes 
@@ -134,12 +138,14 @@ processHandwriting <- function(img, dims) {
   graphdf$from <- indices[graphdf$Var1]
   # get the index in thinned image of the neighbor pixel for each pixel
   graphdf$to <- graphdf$from + c(-1, dims[1] - 1, dims[1], dims[1] + 1, 1, 1 - dims[1], -dims[1], -1 - dims[1])[graphdf$Var2]
-  # Manhattan distance between each pixel and its neighbor
+  # Manhattan distance between each pixel and its neighbor. neighbors to the
+  # top, right, bottom, and left are 1 in distance and neighbors on the
+  # diagonals are 2 in distance.
   graphdf$man_dist <- rep(c(1, 2), 4)[graphdf$Var2]
   # Euclidean distance between each pixel and its neighbor
   graphdf$euc_dist <- c(1, sqrt(2), 1, sqrt(2), 1, sqrt(2), 1, sqrt(2))[graphdf$Var2]
   graphdf$pen_dist <- c(1, 3, 1, 3, 1, 3, 1, 3)[graphdf$Var2]
-
+  
   # Step 1: make matrix of neighborhoods for each pixel in the thinned writing: 
   #   each row corresponds to a pixel in the thinned writing
   #   each column corresponds to a location in the pixel's neighborhood
@@ -172,7 +178,7 @@ processHandwriting <- function(img, dims) {
   graphdf$to <- as.character(format(graphdf$to, scientific = FALSE, trim = TRUE))
   # drop Var1, Var2, and value columns
   graphdf <- subset(graphdf, select = c(from, to, man_dist, euc_dist, pen_dist))
-
+  
   # build undirected graph with vertices=indices of the thinned writing and edges listed in graphdf
   skel_graph <- igraph::graph_from_data_frame(d = graphdf, vertices = as.character(format(indices, scientific = FALSE, trim = TRUE)), directed = FALSE)
   # build undirected graph with vertices=indices of the thinned writing and edges listed in graphdf0
@@ -184,7 +190,7 @@ processHandwriting <- function(img, dims) {
   # color vertex as 1 if vertex is a node, otherwise color as 0
   V(skel_graph)$color <- ifelse(V(skel_graph)$name %in% nodeList, 1, 0)
   V(skel_graph0)$color <- ifelse(V(skel_graph0)$name %in% nodeList, 1, 0)
-
+  
   # if a node isn't connected, assign it to the terminal nodes list
   terminalNodes <- nodeList[!(nodeList %in% nodeConnections)]
   
@@ -194,8 +200,10 @@ processHandwriting <- function(img, dims) {
                       v = as.character(format(nodeList, scientific = FALSE, trim = TRUE)), 
                       to = as.character(format(nodeList, scientific = FALSE, trim = TRUE)), 
                       weights = E(skel_graph0)$nodeOnlyDist)
+  # Create adjacency matrix with 1 if the distance is 1 or 2 and 0 otherwise
   adj0 <- ifelse(dists0 == 1 | dists0 == 2, 1, 0)
-
+  
+  # And merging them ----
   message("and merging them...")
   emergencyBreak <- 100
   while (TRUE) {
@@ -205,32 +213,35 @@ processHandwriting <- function(img, dims) {
                            v = as.character(format(nodeList, scientific = FALSE, trim = TRUE)), 
                            to = as.character(format(nodeList, scientific = FALSE, trim = TRUE)), 
                            weights = NA)
+    # set all values on the diagonal and below the diagonal to 0
     distsFull[!upper.tri(distsFull)] <- 0
     # flag nodes that are only 1 or 2 edges apart
     nodesToMerge <- which(distsFull <= 2 & distsFull > 0)
     rNodes <- ((nodesToMerge - 1) %% length(nodeList)) + 1
     cNodes <- ((nodesToMerge - 1) %/% length(nodeList)) + 1
     mergeSets <- cbind(nodeList[rNodes], nodeList[cNodes])
-    # add column: 0=one or both nodes is terminal, 1=neigther is terminal
+    # add column: 1=neither node is terminal, 0 otherwise
     mergeSets <- cbind(mergeSets, apply(mergeSets, 1, function(x) {
       all(!(x %in% terminalNodes))
     }))
+    # keep rows of nodes where neither node is terminal
     mergeSets <- mergeSets[mergeSets[, 3] == 1, c(1, 2)]
     mergeSets <- matrix(mergeSets, ncol = 2)
     
     # end while loop if no nodes need to be merged
     if (dim(mergeSets)[1] == 0) break
-
-    if (anyDuplicated(c(mergeSets)) > 0) {
-      duplicates <- which(mergeSets %in% mergeSets[apply(matrix(duplicated(c(mergeSets)), ncol = 2), 1, any)])
-      rduplicates <- ((duplicates - 1) %% dim(mergeSets)[1]) + 1
-      duplicateDists <- distsFull[matrix(as.character(format(mergeSets[rduplicates, ], scientific = FALSE, trim = TRUE)), ncol = 2)]
-    }
+    
+    # if (anyDuplicated(c(mergeSets)) > 0) {
+    #   duplicates <- which(mergeSets %in% mergeSets[apply(matrix(duplicated(c(mergeSets)), ncol = 2), 1, any)])
+    #   rduplicates <- ((duplicates - 1) %% dim(mergeSets)[1]) + 1
+    #   duplicateDists <- distsFull[matrix(as.character(format(mergeSets[rduplicates, ], scientific = FALSE, trim = TRUE)), ncol = 2)]
+    # }
     newNodes <- findMergeNodes(skel_graph, mergeSets)
+    # combine nodes that were not in the mergeSets and the list of new nodes
     nodeList <- unique(c(nodeList[!(nodeList %in% c(mergeSets[, c(1, 2)]))], newNodes))
-
+    
     # At this point have the updated nodeList, if we wanted to break it letter by letter we would do that here
-
+    
     ### Migrate connections from original nodes to new nodes
     toDelete <- NULL
     nRowCol <- dim(adj0)[1]
@@ -239,10 +250,10 @@ processHandwriting <- function(img, dims) {
       whichRowCol <- which(colnames(adj0) %in% format(mergeSets[i, c(1, 2)], scientific = FALSE, trim = TRUE))
       newConnectivities <- apply(matrix(adj0[whichRowCol, ], nrow = length(whichRowCol)), 2, function(x) x[1] == 1 | x[2] == 1)
       newConnectivities[is.na(newConnectivities)] <- 0
-
+      
       toAdd <- dim(adj0)[1] + 1
       toDelete <- c(toDelete, which(rownames(adj0) %in% format(mergeSets[i, c(1, 2)], scientific = FALSE, trim = TRUE)))
-
+      
       adj0 <- rbind(cbind(adj0, 0), 0)
       adj0[, toAdd] <- c(newConnectivities, 0)
       adj0[toAdd, ] <- c(newConnectivities, 0)
@@ -252,36 +263,41 @@ processHandwriting <- function(img, dims) {
     if (length(toDelete) > 0) {
       adj0 <- as.matrix(adj0[, -toDelete])[-toDelete, ]
     }
-
+    
     emergencyBreak <- emergencyBreak - 1
     if (emergencyBreak == 0) {
       break()
     }
   }
-
+  
+  # update graphdf0 with nodeOnlyDist=1 if one or more of the edge's vertices is a node and 0.00001 otherwise
   graphdf0 <- as_data_frame(skel_graph0)
   graphdf0$nodeOnlyDist <- ifelse(graphdf0$from %in% nodeList | graphdf0$to %in% nodeList, 1, 0.00001)
   skel_graph0 <- graph_from_data_frame(graphdf0, directed = FALSE)
-
+  
+  # Set the diagonal and below of the adjacency matrix to 0
   adj0[lower.tri(adj0)] <- 0
   adj.m <- melt(adj0)
   adj.m <- subset(adj.m, value == 1)
   names(adj.m) <- c("from", "to", "value")
-
+  
+  # Finding direct paths ----
   message("Finding direct paths...", appendLF = FALSE)
-  pathList <- AllUniquePaths(adj.m, skel_graph, skel_graph0)
-
+  pathList <- AllUniquePaths(adj.m, skel_graph0)
+  
+  # And loops ----
   message("and loops...")
   loopList <- getLoops(nodeList, skel_graph, skel_graph0, pathList, dim(img))
-
+  
   allPaths <- append(pathList, loopList)
-
+  
   graphdf0 <- as_data_frame(skel_graph0)
   graphdf0$nodeOnlyDist <- ifelse(graphdf0$from %in% nodeList | graphdf0$to %in% nodeList, 1, 0)
   skel_graph0 <- graph_from_data_frame(graphdf0, directed = FALSE)
-
+  
+  # Looking for graph break points ----
   # Nominate and check candidate breakpoints
-  message("Looking for letter break points...", appendLF = FALSE)
+  message("Looking for graph break points...", appendLF = FALSE)
   hasTrough <- rep(FALSE, length(pathList))
   troughNodes <- c()
   candidateNodes <- c()
@@ -290,12 +306,21 @@ processHandwriting <- function(img, dims) {
     # Look for troughs in edges.
     tempPath <- pathList[[i]]
     if (length(tempPath) > 10) {
+      # get the row number of each vertex in the path
       rows <- ((tempPath - 1) %% dims[1]) + 1
       for (j in 5:(length(rows) - 4))
-      {
+      { # if there is at least one vertex before j and at least on after j that are higher than j
         if (any(rows[1:(j - 1)] < rows[j] - 1) & any(rows[(j + 1):length(rows)] < rows[j] - 1)) {
+          # of all the vertices that are 2 or more rows higher than j (in the image) and to the left of 
+          # j (in the rows vector) find the index of the vertex (in rows[1:(j-1)]) that is closest to j from
+          # the left
           lowerEnd <- max(which(rows[1:(j - 1)] < rows[j] - 1))
+          # of all the vertices that are 2 or more rows higher than j (in the image) and to the right of 
+          # j (in the rows vector) find the index of the vertex (in rows[(j+1):length(rows)]) that is closest to j from
+          # the right
           upperEnd <- min(which(rows[(j + 1):length(rows)] < rows[j] - 1))
+          # if none of the vertices between the lowerEnd and upperEnd are lower than vertex j, assign j
+          # as a trough node
           if (!any(rows[lowerEnd:(j + upperEnd)] > rows[j])) {
             troughNodes <- c(troughNodes, tempPath[j])
             hasTrough[i] <- TRUE
@@ -304,46 +329,64 @@ processHandwriting <- function(img, dims) {
       }
     }
     if (hasTrough[i] == FALSE) {
+      # add the middle node to the candidate list
       candidateNodes <- c(candidateNodes, tempPath[ceiling(length(tempPath) / 2)])
     }
   }
-  breaks <- which((((troughNodes[-1] - 1) %% dims[1]) + 1) != (((troughNodes[-length(troughNodes)] - 1) %% dims[1]) + 1) |
-    ((((troughNodes[-1] - 1) %/% dims[1]) + 1) != (((troughNodes[-length(troughNodes)] - 1) %/% dims[1])) &
-      (((troughNodes[-1] - 1) %/% dims[1])) != (((troughNodes[-length(troughNodes)] - 1) %/% dims[1]) + 1)))
+  # find the trough nodes where: (1) the row of the trough node is not equal to
+  # the row of the next trough node, or (2) the column of the trough node is
+  # different than the column of the next trough node minus 1, and (3) the column of the 
+  # trough node minus 1 is not equal to the column of the text trough node.
+  breaks <- which(i_to_r(troughNodes[-1], dims[1]) != i_to_r(troughNodes[-length(troughNodes)], dims[1]) |
+                    i_to_c(troughNodes[-1], dims[1]) != i_to_c(troughNodes[-length(troughNodes)], dims[1]) - 1 &
+                    i_to_c(troughNodes[-1], dims[1]) - 1 != i_to_c(troughNodes[-length(troughNodes)], dims[1]))
   breaks <- c(1, breaks, length(troughNodes))
   candidateNodes <- c(candidateNodes, troughNodes[ceiling((breaks[-1] + breaks[-length(breaks)]) / 2)])
-
+  
+  # And discarding bad ones ----
   message("and discarding bad ones...")
-
-  goodBreaks <- checkBreakPoints(candidateNodes = candidateNodes, allPaths = pathList, nodeGraph = getNodeGraph(pathList, nodeList), terminalNodes = terminalNodes, dims)
+  
+  # create a graph with vertices for each node and edges between the first and last pixel / vertex in each path
+  nodeGraph <- getNodeGraph(pathList, nodeList)
+  # flag candidates as good if none of the following are true: (1) there is more
+  # than one route between the first and last nodes in the path containing the candidate (2) the path
+  # contains a terminal node (3) the candidate break point is within 4 of the
+  # first or last node in the path (4) the path contains 10 or fewer
+  # vertices.
+  goodBreaks <- checkBreakPoints(candidateNodes = candidateNodes, allPaths = pathList, nodeGraph = nodeGraph, terminalNodes = terminalNodes, dims)
   preStackBreaks <- candidateNodes[goodBreaks]
-
+  # list the break(s) in each path or 'integer(0)' if the path does not contain a break
   pathsWithBreaks <- lapply(allPaths, function(x) {
     which(x %in% preStackBreaks)
   })
+  # number of breaks per path
   breaksPerPath <- unlist(lapply(pathsWithBreaks, length))
+  # update the list of breaks: if a path has more than one break, take the average of the breaks (rounding up) and
+  # remove the original breaks so that each path has either 0 or 1 break.
   for (i in which(breaksPerPath > 1))
-  {
+  { # if current path has more than one break, average the breaks and round up
     newBreak <- floor(mean(which(allPaths[[i]] %in% preStackBreaks)))
+    # drop pre stack breaks in current path from the pre stack breaks list
     preStackBreaks <- preStackBreaks[which(!(preStackBreaks %in% allPaths[[i]]))]
+    # add "new break" for current path to list
     preStackBreaks <- c(preStackBreaks, allPaths[[i]][newBreak])
   }
-
-
-  message("Isolating letter paths...")
-
+  
+  # Isolating graph paths ----
+  message("Isolating graph paths...")
+  
   ## Break on breakpoints and group points by which letter they fall into. Adjust graph accordingly.
   letterList <- letterPaths(allPaths, skel_graph0, preStackBreaks)
   letters <- letterList[[1]][unlist(lapply(letterList[[1]], length)) > 5]
-
+  
   V(skel_graph0)$letterID <- letterList[[2]]
-  skel_graph0 <- delete.vertices(skel_graph0, V(skel_graph0)[which(V(skel_graph0)$letterID %in% which(unlist(lapply(letterList[[1]], length)) <= 5))])
+  skel_graph0 <- igraph::delete_vertices(skel_graph0, V(skel_graph0)[which(V(skel_graph0)$letterID %in% which(unlist(lapply(letterList[[1]], length)) <= 5))])
   V(skel_graph0)$letterID[!is.na(V(skel_graph0)$letterID)] <- as.numeric(as.factor(na.omit(V(skel_graph0)$letterID)))
-
+  
   # Remove breakpoints that shouldn't have broken.
   finalBreaks <- preStackBreaks[!(checkStacking(preStackBreaks, allPaths, letters, skel_graph0, dims))]
   finalBreaks <- finalBreaks[!(checkSimplicityBreaks(finalBreaks, pathList, loopList, letters, skel_graph0, nodeList, terminalNodes, hasTrough, dims))]
-
+  
   pathsWithBreaks <- lapply(allPaths, function(x) {
     which(x %in% preStackBreaks)
   })
@@ -357,15 +400,15 @@ processHandwriting <- function(img, dims) {
           E(skel_graph0, P = format(allPaths[[i]][c(newNodes - 2, newNodes - 1)], scientific = FALSE, trim = TRUE))$nodeOnlyDist <- 1
           E(skel_graph0, P = format(allPaths[[i]][c(newNodes + 1, newNodes + 2)], scientific = FALSE, trim = TRUE))$nodeOnlyDist <- 1
         }, error = function(e) {
-
+          
         }
       )
-
+      
       newNodes <- c(newNodes - 1, newNodes + 1)
       nodeList <- c(nodeList, allPaths[[i]][newNodes])
       allPaths[[i]] <- list(allPaths[[i]][1:(newNodes[1])], allPaths[[i]][(newNodes[2]):length(allPaths[[i]])])
     }
-
+    
     # letterIDs = range(V(skel_graph0)$letterID[names(V(skel_graph0)) %in% format(allPaths[[i]], scientific = FALSE, trim = TRUE)], na.rm = TRUE)
     # Had to break this above line into this to stop a range(numeric(0)) error from happening
     skelInPaths <- V(skel_graph0)$letterID[names(V(skel_graph0)) %in% format(allPaths[[i]], scientific = FALSE, trim = TRUE)]
@@ -374,24 +417,28 @@ processHandwriting <- function(img, dims) {
     } else {
       letterIDs <- range(skelInPaths, na.rm = TRUE)
     }
-
+    
     V(skel_graph0)$letterID[which(V(skel_graph0)$letterID == letterIDs[2])] <- letterIDs[1]
     V(skel_graph0)$letterID[which(names(V(skel_graph0)) %in% format(allPaths[[i]][newNodes], scientific = FALSE, trim = TRUE))] <- letterIDs[1]
   }
-
+  
   allPaths <- lapply(rapply(allPaths, enquote, how = "unlist"), eval)
-
+  
+  # Organizing letters ----
   message("Organizing letters...")
   letters <- organize_letters(skel_graph0)
-
+  
+  # Creating letter lists ----
   message("Creating letter lists...")
   letterList <- create_letter_lists(allPaths, letters, nodeList, nodeConnections, terminalNodes, dims)
-
+  
+  # Adding character features ----
   message("Adding character features...")
   letterList <- add_character_features(img, letterList, letters, dims)
-
+  
+  # Document processing complete ----
   message("Document processing complete.\n")
-
+  
   return(list(nodes = nodeList, connectingNodes = nodeConnections, terminalNodes = terminalNodes, breakPoints = sort(finalBreaks), letterList = letterList))
 }
 
@@ -409,18 +456,18 @@ processHandwriting <- function(img, dims) {
 #' @noRd
 add_character_features <- function(img, letterList, letters, dims) {
   featureSets <- extract_character_features(img, letterList, dims)
-
+  
   for (i in 1:length(letters))
   {
     letterList[[i]]$characterFeatures <- featureSets[[i]]
   }
-
+  
   letterPlaces <- matrix(unlist(lapply(featureSets, FUN = function(x) {
     c(x$line_number, x$order_within_line)
   })), ncol = 2, byrow = TRUE)
   letterOrder <- order(letterPlaces[, 1], letterPlaces[, 2])
   letterList <- letterList[letterOrder]
-
+  
   return(letterList)
 }
 
@@ -428,45 +475,53 @@ add_character_features <- function(img, letterList, letters, dims) {
 #'
 #' Internal function for getting a list of all non loop paths in a writing sample.
 #'
-#' @param adj adjacent matrix
-#' @param graph first skeletonized graph
-#' @param graph0 second skeletonized graph
+#' @param adj adjacent matrix of nodes
+#' @param graph0 skeletonized graph
 #' @return a list of all non loop paths
 #' @noRd
-AllUniquePaths <- function(adj, graph, graph0) {
-  # Gets all paths that are not loops
-  # paths = apply(adj, 1, LooplessPaths, graph = graph, graph0 = graph0)
+AllUniquePaths <- function(adj, graph0) {
   paths <- list()
   if (dim(adj)[1] == 0) {
     return(NULL)
   }
-  #
+  
   for (i in 1:dim(adj)[1])
   {
     fromNode <- as.character(format(adj[i, 1], scientific = FALSE, trim = TRUE))
     toNode <- as.character(format(adj[i, 2], scientific = FALSE, trim = TRUE))
 
-    while (shortest.paths(graph0, v = fromNode, to = toNode, weights = E(graph0)$nodeOnlyDist) < 3 & shortest.paths(graph0, v = fromNode, to = toNode, weights = E(graph0)$nodeOnlyDist) >= 1) {
+    # calculate distances of shortest path between fromNode and toNode
+    dists <- distances(graph0, v = fromNode, to = toNode, weights = E(graph0)$nodeOnlyDist)
+    # for paths from node to node that don't go through another node?
+    while (dists < 3 & dists >= 1) {
+      # CALCULATE STEP:
+      # get shortest path between fromNode and toNode
       shortest <- shortest_paths(graph0, from = fromNode, to = toNode, weights = E(graph0)$nodeOnlyDist)
+      # count vertices in shortest path, including fromNode and toNode
       len <- length(unlist(shortest[[1]]))
+      # add the shortest path to the list
       paths <- c(paths, list(as.numeric(names(shortest$vpath[[1]]))))
+      
+      # UPDATE STEP: if there are more than two vertices in the path
       if (len > 2) {
-        graph <- delete.edges(graph, paste0(names(shortest$vpath[[1]])[len %/% 2], "|", names(shortest$vpath[[1]])[len %/% 2 + 1]))
-        graph0 <- delete.edges(graph0, paste0(names(shortest$vpath[[1]])[len %/% 2], "|", names(shortest$vpath[[1]])[len %/% 2 + 1]))
+        graph0 <- igraph::delete_edges(graph0, paste0(names(shortest$vpath[[1]])[len %/% 2], "|", names(shortest$vpath[[1]])[len %/% 2 + 1]))
       } else if (len == 2) {
-        graph0 <- delete.edges(graph0, paste0(names(shortest$vpath[[1]])[1], "|", names(shortest$vpath[[1]])[2]))
+        graph0 <- igraph::delete_edges(graph0, paste0(names(shortest$vpath[[1]])[1], "|", names(shortest$vpath[[1]])[2]))
       } else {
         stop("There must be some mistake. Single node should have nodeOnlyDist path length of 0.")
       }
+      # recalculate shortest path distance on updated graph
+      dists <- distances(graph0, v = fromNode, to = toNode, weights = E(graph0)$nodeOnlyDist)
     }
   }
-
+  
   return(paths)
 }
 
 #' checkBreakPoints
 #'
-#' Internal function called by processHandwriting that eliminates breakpoints based on rules to try to coherently separate letters.
+#' Internal function called by processHandwriting that eliminates breakpoints
+#' based on rules to try to coherently separate letters.
 #'
 #' @param candidateNodes possible breakpoints
 #' @param allPaths list of paths
@@ -479,15 +534,16 @@ AllUniquePaths <- function(adj, graph, graph0) {
 checkBreakPoints <- function(candidateNodes, allPaths, nodeGraph, terminalNodes, dims) {
   # Check rules for candidate breakpoints
   breakFlag <- rep(TRUE, length(candidateNodes))
-
+  
   for (i in 1:length(allPaths))
-  {
+  { # create character vector of each vertex in path i
     tempPath <- format(allPaths[[i]], scientific = FALSE, trim = TRUE)
+    # check if path i contains candidate nodes
     nodeChecks <- which(candidateNodes %in% tempPath)
-    tempNodeGraph <- delete.edges(nodeGraph, paste0(tempPath[1], "|", tempPath[length(tempPath)]))
+    tempNodeGraph <- igraph::delete_edges(nodeGraph, paste0(tempPath[1], "|", tempPath[length(tempPath)]))
 
     if (distances(tempNodeGraph, v = tempPath[1], to = tempPath[length(tempPath)]) < Inf) {
-      # No breaking on multiple paths between nodes.
+      # No breaking on multiple paths between nodes. 
       breakFlag[nodeChecks] <- FALSE
     } else if (any(tempPath %in% terminalNodes)) {
       # No break if path has an endpoint
@@ -497,7 +553,7 @@ checkBreakPoints <- function(candidateNodes, allPaths, nodeGraph, terminalNodes,
       breakFlag[nodeChecks[which(candidateNodes[nodeChecks] <= 5 | candidateNodes[nodeChecks] >= length(tempPath) - 4)]] <- FALSE
     }
   }
-
+  
   return(breakFlag)
 }
 
@@ -525,19 +581,19 @@ checkSimplicityBreaks <- function(candidateBreaks, pathList, loopList, letters,
     if (length(nodestoCheck) >= 1) {
       if (!hasTrough[i]) {
         pathIndex <- which(tempPath == candidateBreaks[nodestoCheck])
-
+        
         borderLetters <- c(
           V(nodeGraph0)$letterID[which(V(nodeGraph0)$name == tempPath[pathIndex - 1])],
           V(nodeGraph0)$letterID[which(V(nodeGraph0)$name == tempPath[pathIndex + 1])]
         )
         left <- letters[[borderLetters[1]]]
         right <- letters[[borderLetters[2]]]
-
+        
         nodesOnLeft <- sum(nodeList %in% left)
         nodesOnRight <- sum(nodeList %in% right)
         terminalLeft <- sum(terminalNodes %in% left)
         terminalRight <- sum(terminalNodes %in% right)
-
+        
         if (nodesOnLeft == 3 & nodesOnRight == 3 & terminalLeft == 2 & terminalRight == 2) {
           pathsOnLeft <- length(pathLetterAssociate(c(pathList, loopList), left))
           pathsOnRight <- length(pathLetterAssociate(c(pathList, loopList), right))
@@ -564,7 +620,7 @@ checkSimplicityBreaks <- function(candidateBreaks, pathList, loopList, letters,
 #' @noRd
 checkStacking <- function(candidateBreaks, allPaths, letters, nodeGraph0, dims) {
   stackPtFlag <- rep(FALSE, length(candidateBreaks))
-
+  
   for (i in 1:length(allPaths))
   {
     tempPath <- allPaths[[i]]
@@ -576,7 +632,7 @@ checkStacking <- function(candidateBreaks, allPaths, letters, nodeGraph0, dims)
         stackPtFlag[nodeChecks] <- TRUE
       } else {
         pathIndex <- which(tempPath == candidateBreaks[nodeChecks])
-
+        
         borderLetters <- c(
           V(nodeGraph0)$letterID[which(V(nodeGraph0)$name == tempPath[pathIndex - 1])],
           V(nodeGraph0)$letterID[which(V(nodeGraph0)$name == tempPath[pathIndex + 1])]
@@ -585,7 +641,7 @@ checkStacking <- function(candidateBreaks, allPaths, letters, nodeGraph0, dims)
         gr1Rows <- ((gr1 - 1) %% dims[1]) + 1
         gr2 <- letters[[borderLetters[2]]]
         gr2Rows <- ((gr2 - 1) %% dims[1]) + 1
-
+        
         # Call a break a stack point if the overlap between the bordering letters is
         # less than 10% of the total range of the combined letters.
         if (is.null(gr1) || is.null(gr2)) {
@@ -612,15 +668,15 @@ create_letter_lists <- function(allPaths, letters, nodeList, nodeConnections, te
   nodesinGraph <- replicate(length(letters), list(NA))
   connectingNodesinGraph <- replicate(length(letters), list(NA))
   terminalNodesinGraph <- replicate(length(letters), list(NA))
-
+  
   for (i in 1:length(letters))
   {
     nodesinGraph[[i]] <- letters[[i]][which(letters[[i]] %in% nodeList)]
     connectingNodesinGraph[[i]] <- letters[[i]][which(letters[[i]] %in% nodeConnections)]
     terminalNodesinGraph[[i]] <- letters[[i]][which(letters[[i]] %in% terminalNodes)]
   }
-
-
+  
+  
   letterList <- replicate(length(letters), list(path = NA, nodes = NA), simplify = FALSE)
   for (i in 1:length(letters))
   {
@@ -628,12 +684,12 @@ create_letter_lists <- function(allPaths, letters, nodeList, nodeConnections, te
     # letterList[[i]]$nodes = nodesinGraph[[i]][nodeOrder[[i]]]
     letterList[[i]]$allPaths <- pathLetterAssociate(allPaths, letters[[i]])
   }
-
+  
   letterAdj <- list()
   nodeOrder <- replicate(list(), n = length(letters))
   decCode <- rep("", length(letters))
   connectivityScores <- replicate(list(), n = length(letters))
-
+  
   getConnectivity <- function(pathEndings, nodesSingle) {
     res <- rep(NA, length(nodesSingle))
     for (j in 1:length(nodesSingle))
@@ -642,21 +698,21 @@ create_letter_lists <- function(allPaths, letters, nodeList, nodeConnections, te
     }
     return(res)
   }
-
+  
   for (i in 1:length(letters))
   {
     if (length(nodesinGraph[[i]]) > 0) {
       letterList[[i]]$adjMatrix <- matrix(0, ncol = length(nodesinGraph[[i]]), nrow = length(nodesinGraph[[i]]))
-
+      
       pathStarts <- unlist(lapply(letterList[[i]]$allPaths, function(x) x[1]))
       pathEnds <- unlist(lapply(letterList[[i]]$allPaths, function(x) x[length(x)]))
-
+      
       connectivityScores[[i]] <- getConnectivity(pathEndings = c(pathStarts, pathEnds), nodesSingle = nodesinGraph[[i]])
-
+      
       nodeOrder[[i]] <- getNodeOrder(letters[[i]], nodesinGraph[[i]], connectivityScores[[i]], dims)
-
+      
       nodeSet <- nodesinGraph[[i]][order(nodeOrder[[i]])]
-
+      
       warn <- FALSE
       for (j in 1:length(pathStarts))
       {
@@ -666,7 +722,7 @@ create_letter_lists <- function(allPaths, letters, nodeList, nodeConnections, te
         } else {
           pathStarts[j] <- which(nodeSet == pathStarts[j])
         }
-
+        
         if (!(pathEnds[j] %in% nodeSet)) {
           warning(paste0("Maybe a loop that didn't merge with node. letterList[[", i, "]]"))
           warn <- TRUE
@@ -677,7 +733,7 @@ create_letter_lists <- function(allPaths, letters, nodeList, nodeConnections, te
       if (warn) {
         next
       }
-
+      
       letterList[[i]]$adjMatrix[cbind(pathStarts, pathEnds)] <- 1
       letterList[[i]]$adjMatrix[cbind(pathEnds, pathStarts)] <- 1
       binCode <- t(letterList[[i]]$adjMatrix)[!upper.tri(letterList[[i]]$adjMatrix)]
@@ -701,7 +757,7 @@ create_letter_lists <- function(allPaths, letters, nodeList, nodeConnections, te
       letterList[[i]]$letterCode <- "A"
     }
   }
-
+  
   return(letterList)
 }
 
@@ -805,9 +861,9 @@ findMergeNodes <- function(skel_graph, mergeMat) {
 #' @noRd
 getLoops <- function(nodeList, graph, graph0, pathList, dims) {
   vertexNames <- names(V(graph0))
-
+  
   fullGraph0 <- graph0
-
+  
   used <- unlist(lapply(pathList, function(x) {
     x[-c(1, length(x))]
   }))
@@ -821,27 +877,27 @@ getLoops <- function(nodeList, graph, graph0, pathList, dims) {
     unusedAdj[which(unused %in% nodeList), which(unused %in% nodeList)] <- 0
   }
   unusedGraph <- graph_from_adjacency_matrix(unusedAdj, mode = "undirected")
-
+  
   graph0 <- intersection(graph0, unusedGraph, keep.all.vertices = TRUE)
   graph <- intersection(graph, graph0, byname = TRUE, keep.all.vertices = TRUE)
   check <- unused[degree(graph0, as.character(format(unused, scientific = FALSE, trim = TRUE))) > 1]
   check <- check[which(check %in% nodeList)]
-
+  
   loopList <- list()
-
+  
   tryCatch(
     expr = {
       neighbors <- neighborhood(graph, nodes = as.character(check))
-
+      
       if (any(unlist(lapply(neighbors, length)) > 3)) {
         warning("At least 1 of the nodes in the potential loops has more than 2 neighbors after removal of the connections. Try again! \nThe nodes in question are: \n", dput(names(neighbors)[which(unlist(lapply(neighbors, length)) > 3)]))
       }
-
+      
       ## Get paths that start and end at the same point, where that point is a node in nodeList
       if (length(neighbors) > 0) {
         for (i in 1:length(neighbors)) {
           neigh <- as.numeric(names(neighbors[[i]]))
-          graph <- delete.edges(graph, paste0(neigh[1], "|", neigh[2]))
+          graph <- igraph::delete_edges(graph, paste0(neigh[1], "|", neigh[2]))
           if (distances(graph, v = as.character(neigh[1]), to = as.character(neigh[2])) < Inf) {
             newPath <- as.numeric(names(unlist(shortest_paths(graph, from = format(neigh[1], scientific = FALSE), to = format(neigh[2], scientific = FALSE), weights = E(graph)$pen_dist)$vpath)))
             loopList <- append(loopList, list(c(newPath, newPath[1])))
@@ -853,16 +909,16 @@ getLoops <- function(nodeList, graph, graph0, pathList, dims) {
       message("Error in loops... skipping...")
     }
   )
-
+  
   ## Eliminate loop paths that we have found and find ones that dont have vertex on the loop. This is caused by combining of nodes that are close together.
   used <- as.numeric(unique(c(unlist(pathList), unlist(loopList))))
   unused <- as.numeric(vertexNames)[which(!(as.numeric(vertexNames) %in% used))]
   remaining0 <- induced_subgraph(graph0, vids = format(c(unused, nodeList), scientific = FALSE, trim = TRUE))
   numNeighbors <- lapply(neighborhood(remaining0, nodes = V(remaining0)), length)
   remaining0 <- induced_subgraph(remaining0, vids = V(remaining0)[numNeighbors > 1])
-
+  
   roots <- format(nodeList[which(nodeList %in% names(V(remaining0)))], scientific = FALSE, trim = TRUE)
-
+  
   if (length(roots) > 0) {
     for (i in 1:length(roots))
     {
@@ -871,19 +927,19 @@ getLoops <- function(nodeList, graph, graph0, pathList, dims) {
       loopList <- append(loopList, list(as.numeric(c(loopPart1, names(loopPart2)))))
     }
   }
-
+  
   ## Now get loops that are more difficult. They are close to nodes, but separated by paths already found previously. Have to dig a little further.
   remaining0 <- induced_subgraph(graph0, vids = format(unused, scientific = FALSE, trim = TRUE))
   used <- as.numeric(unique(c(unlist(pathList), unlist(loopList))))
   unused <- as.numeric(vertexNames)[which(!(as.numeric(vertexNames) %in% used))]
   if (length(unused) > 0) {
     ends <- lapply(neighborhood(fullGraph0, order = 2, nodes = format(unused, scientific = FALSE, trim = TRUE)), function(x) nodeList[which(format(nodeList, scientific = FALSE, trim = TRUE) %in% names(x))])
-
+    
     roots <- format(unique(unlist(ends)), scientific = FALSE, trim = TRUE)
     if (length(roots) > 0) {
       ends <- neighborhood(fullGraph0, order = 2, nodes = roots)
       ends <- unlist(lapply(ends, function(x) names(x)[which(names(x) %in% format(unused, scientific = FALSE, trim = TRUE))][1]))
-
+      
       for (i in 1:length(roots))
       {
         loopPart1 <- names(na.omit(dfs(remaining0, ends[i], unreachable = FALSE)$order))
@@ -893,20 +949,20 @@ getLoops <- function(nodeList, graph, graph0, pathList, dims) {
       }
     }
   }
-
-
+  
+  
   ## And a little deeper
   remaining0 <- induced_subgraph(graph0, vids = format(unused, scientific = FALSE, trim = TRUE))
   used <- as.numeric(unique(c(unlist(pathList), unlist(loopList))))
   unused <- as.numeric(vertexNames)[which(!(as.numeric(vertexNames) %in% used))]
   if (length(unused) > 0) {
     ends <- lapply(neighborhood(fullGraph0, order = 3, nodes = format(unused, scientific = FALSE, trim = TRUE)), function(x) nodeList[which(format(nodeList, scientific = FALSE, trim = TRUE) %in% names(x))])
-
+    
     roots <- format(unique(unlist(ends)), scientific = FALSE, trim = TRUE)
     if (length(roots) > 0) {
       ends <- neighborhood(fullGraph0, order = 3, nodes = roots)
       ends <- unlist(lapply(ends, function(x) names(x)[which(names(x) %in% format(unused, scientific = FALSE, trim = TRUE))][1]))
-
+      
       for (i in 1:length(roots))
       {
         loopPart1 <- names(na.omit(dfs(remaining0, ends[i], unreachable = FALSE)$order))
@@ -916,19 +972,19 @@ getLoops <- function(nodeList, graph, graph0, pathList, dims) {
       }
     }
   }
-
+  
   ## All that remains now is perfect loops. Start and end at same point with no intersections or end points.
   remaining0 <- induced_subgraph(remaining0, vids = V(remaining0)[!(names(V(remaining0)) %in% unlist(loopList))])
   while (TRUE) {
     if (length(V(remaining0)) > 0) {
       perfectLoop <- names(na.omit(dfs(remaining0, V(remaining0)[1], unreachable = FALSE)$order))
-      remaining0 <- delete.vertices(remaining0, v = perfectLoop)
+      remaining0 <- igraph::delete_vertices(remaining0, v = perfectLoop)
       loopList <- append(loopList, list(as.numeric(c(perfectLoop, perfectLoop[1]))))
     } else {
       break
     }
   }
-
+  
   return(loopList)
 }
 
@@ -957,7 +1013,7 @@ getNodes <- function(indices, dims) {
   nodes <- matrix(1, dims[1], dims[2])
   # add nodes to matrix as 0
   nodes[indices] <- ifelse(changeCount == 1 | changeCount >= 3, 0, 1)
-
+  
   ## If there is a 2x2 block in the thinned image and none of those pixels are nodes, make one of them a node.
   ## All will have connectivity of 2. Choose pixel with most neighbors as node. Also make opposite diagonal pixel a node.
   ## When nodes are combined later this will form 1 node that absorbs all connections.
@@ -990,15 +1046,17 @@ getNodes <- function(indices, dims) {
   nodes2by2 <- t(apply(img.m, 1, FUN = node2by2fill, img = img))
   # add 2x2 nodes to nodes matrix as 0
   nodes[c(nodes2by2[apply(nodes2by2, 1, function(x) {all(!is.na(x))}), ])] <- 0
-
+  
   return(list(which(nodes == 0), 
               c(indices[changeCount >= 3], c(nodes2by2[apply(nodes2by2, 1, function(x) {all(!is.na(x))}), ]))
-              ))
+  ))
 }
 
 #' getNodeGraph
 #'
-#' Internal function for creating a graph from a path list and node list.
+#' Internal function for creating a graph from a path list and node list. More specifically,
+#' create a graph with a vertex for each node in nodeList. Then add an edge between the first and last
+#' node (pixel / vertex) in each path in allPaths.
 #'
 #' @param allPaths list of paths
 #' @param nodeList list of nodes
@@ -1009,7 +1067,7 @@ getNodeGraph <- function(allPaths, nodeList) {
   nodeGraph <- add_vertices(nodeGraph, length(nodeList), name = format(nodeList, scientific = FALSE, trim = TRUE))
   for (i in 1:length(allPaths))
   {
-    nodeGraph <- add.edges(nodeGraph, format(c(allPaths[[i]][1], allPaths[[i]][length(allPaths[[i]])]), scientific = FALSE, trim = TRUE))
+    nodeGraph <- igraph::add_edges(nodeGraph, format(c(allPaths[[i]][1], allPaths[[i]][length(allPaths[[i]])]), scientific = FALSE, trim = TRUE))
   }
   return(nodeGraph)
 }
@@ -1035,15 +1093,15 @@ getNodeOrder <- function(letter, nodesInGraph, nodeConnectivity, dims) {
     diff <- c(vecs[1, 1] - vecs[2, 1], vecs[2, 2] - vecs[1, 2])
     return(atan2(diff[1], diff[2]))
   }
-
+  
   if (length(nodesInGraph) == 0) {
     return(nodesInGraph)
   } else {
     nodeOrder <- rep(NA, length(nodesInGraph))
-
+    
     nodeCounter <- 1
     maxConnectivity <- max(nodeConnectivity)
-
+    
     for (i in maxConnectivity:1)
     {
       thisTier <- which(nodeConnectivity == i)
@@ -1077,6 +1135,7 @@ getNodeOrder <- function(letter, nodesInGraph, nodeConnectivity, dims) {
   }
 }
 
+
 #' letterPaths
 #'
 #' Internal function that uses existing breakPoint list to assign letters to the nodes in nodeGraph0.
@@ -1088,37 +1147,23 @@ getNodeOrder <- function(letter, nodesInGraph, nodeConnectivity, dims) {
 #' @noRd
 letterPaths <- function(allPaths, nodeGraph0, breakPoints) {
   oldVerts <- V(nodeGraph0)$name
+  # delete break points from the graph
   if (any(as.character(format(breakPoints, scientific = FALSE, trim = TRUE)) %in% names(V(nodeGraph0)))) {
     nodeGraph0 <- delete_vertices(nodeGraph0, v = as.character(format(breakPoints, scientific = FALSE, trim = TRUE)))
   }
-
-  grIDs <- rep(NA, length(V(nodeGraph0)))
-  tryCatch(
-    expr = {
-      dists <- distances(nodeGraph0, v = names(V(nodeGraph0)), to = names(V(nodeGraph0)), weights = E(nodeGraph0)$nodeOnlyDist)
-    }, error = function(e) {
-      message("Unusual amounts of interconnected paths being detected. Do you have crossed out writing in your document?")
-    }
-  )
-
-  vertList <- V(nodeGraph0)$name
-
-  grPaths <- list()
-  i <- 1
-  while (length(vertList) > 0) {
-    tempIDs <- which(dists[which(V(nodeGraph0)$name == vertList[1]), ] < Inf)
-    grPaths <- c(grPaths, list(as.numeric(V(nodeGraph0)$name[tempIDs])))
-    #  grIDs[V(nodeGraph0)$name %in% as.character(grPaths[[i]])] = i
-    grIDs[tempIDs] <- i
-    vertList <- vertList[vertList %in% setdiff(vertList, format(grPaths[[i]], scientific = FALSE, trim = TRUE))]
-    i <- i + 1
-  }
+  
+  # find connected components
+  comps <- igraph::components(nodeGraph0)
   grIDs2 <- rep(NA, length(oldVerts))
-  grIDs2[which(!(oldVerts %in% format(breakPoints, scientific = FALSE, trim = TRUE)))] <- grIDs
-
+  # assign the ID to the original vertices that are not break points
+  grIDs2[which(!(oldVerts %in% format(breakPoints, scientific = FALSE, trim = TRUE)))] <- comps$membership
+  # lists of vertex names grouped by path
+  grPaths <- lapply(1:max(comps$membership), function(x) as.numeric(V(nodeGraph0)$name[comps$membership == x]))
+  
   return(list(grPaths, grIDs2))
 }
 
+
 organize_letters <- function(skel_graph0) {
   letters <- replicate(n = length(na.omit(unique(V(skel_graph0)$letterID))), list())
   strs <- names(V(skel_graph0))
@@ -1163,7 +1208,7 @@ whichNeighbors <- function(coords, img) {
   yesNeighbs <- which(neighbs == 0)
   res <- as.matrix(rep(0, 8), nrow = 1)
   res[yesNeighbs] <- 1
-
+  
   return(res)
 }
 
